using namespace std;

#include <iostream>
#include <unordered_set>
#include <queue>
#include "testlib.h"

long long tene9 = 1000000000;

void calc_depth(const vector<vector<int>> &tree, int v, vector<int> &depth,
                vector<long long> &fullness, vector<long long> &fullness_refl) {
    int d = -1;
    for (int u : tree[v]) {
        calc_depth(tree, u, depth, fullness, fullness_refl);
        d = max(d, depth[u]);
        fullness_refl[v] += fullness_refl[u];
    }
    fullness_refl[v] += fullness[v];
    depth[v] = d + 1;
}

int main(int argc, char *argv[]) {
    registerGen(argc, argv, 1);

    int max_n = opt<int>("n");
    int max_n_w = opt<int>("nw");
    int max_m = opt<int>("m");
    int max_m_w = opt<int>("mw");
    long long max_a = opt<long long>("a");
    long long max_d = opt<long long>("d");
    int comp_w = opt<int>("cmpw");
    int add_sq = opt<int>("sq");
    bool add_bamboo = opt<bool>("bamboo");
    bool do_binary = opt<bool>("binary");

    int n = do_binary + 2 + rnd.wnext(max(0, max_n - 1), max_n_w);
    if (do_binary) { n = pow(2, floor(log2(n))) - 1; }
    int m = 1 + rnd.wnext(max_m, max_m_w);
    printf("%d %d\n", n, m);

    vector<int> parent(n + 1);
    vector <vector<int>> tree(n + 1);
    vector<int> depth(n + 1);

    for (int i = 2; i <= n; ++i) {
        int p = add_bamboo ? (i - 1) : do_binary ? (i / 2) : (1 + rnd.wnext(i - 1, comp_w));
        parent[i] = p;
        tree[p].push_back(i);
    }
    vector<long long> fullness(n + 1, 0);
    vector<long long> fullness_refl(n + 1, 0);
    for (int i = 1; i <= n; ++i) {
        fullness[i] = rnd.next(tree[i].empty() ? 1LL : 0LL, max_a);
    }
    calc_depth(tree, 1, depth, fullness, fullness_refl);

    vector<long long> levels(n + 1);
    queue<int> q;
    levels[1] = do_binary ? (long long) floor(log2(n)) : rnd.next(1LL * depth[1], max(max_d, depth[1] * 1LL));
    q.emplace(1);
    while (!q.empty()) {
        int p = q.front();
        q.pop();
        for (int v : tree[p]) {
            levels[v] = rnd.next((long long) depth[v], levels[p] - 1);
            if (tree[v].empty()) { levels[v] = 0; }
            q.push(v);
        }
    }

    vector<int> edges_perm = rnd.perm(n, 1);
    edges_perm.push_back(0);
    for (int i = n; i >= 1; --i) {
        edges_perm[i] = edges_perm[i - 1];
    }
    edges_perm[0] = 0;
    vector<int> edges_inv_perm(n + 1, 0);
    for (int i = 1; i <= n; ++i) {
        edges_inv_perm[edges_perm[i]] = i;
    }

    for (int i = 1; i <= n; ++i) {
        int was_i = edges_inv_perm[i];
        printf("%d %lld %lld\n", edges_perm[parent[was_i]], levels[was_i], fullness[was_i]);
    }

    for (int i = 0; i < m; ++i) {
        int v = rnd.next(1, n);
        int mode = rnd.next(1, 4);
        long long max_ok_time = (mode == 1) ? fullness_refl[v] :
                                (mode == 2 ? (fullness[v] + (long long) tree[v].size()) :
                                 (mode == 3) ? fullness_refl[v] / (1 + levels[v]) :
                                 (fullness_refl[v] / (1 + levels[1] - levels[v])));
        max_ok_time = max(max_ok_time, fullness[v] + 2); //max_ok_time >= 2
        if (rnd.next(1, add_sq) == 1) {
            max_ok_time = max(max_ok_time, max_a);
        }
        max_ok_time = min(tene9, max_ok_time);
        long long fns = rnd.next(1LL, max_ok_time);
        printf("%d %lld\n", edges_perm[v], fns);

    }

    return 0;
}
